﻿/*! 
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: base_class.h
\brief 通用dll接口定义头文件, 该DLL不依赖Qt。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10 
* please import basedll.dll
*/

#pragma once

#include "base_export.h"
#include <string>
#include <queue>
#include "base_api.h"
#include "customtype.h"

namespace ns_train {

/*!
\brief 动态库调用类。
*/

struct CLibraryData;
class BASE_API CLibrary
{
public:
	CLibrary();
	~CLibrary();

    /**
    * @brief 加载指定的DLL,仅需要提供不带后缀的文件名，比如加载"aaa.dll"，可以写成"aaa"。
    * @param[in] strPath 指定DLL的名称
    * @return true:操作成功, false:失败
    */
    bool open(const char *pszDLLName);
    /**
    * @brief 卸载DLL
    * @return true:操作成功, false:失败
    */
    bool close(void);
    /**
    * @brief 获取DLL中指定函数的地址
    * @param[in] pszFunctionName 指定函数的名称
    * @return true:操作成功, false:失败
    */
	void* getFunction(const char *pszFunctionName);

private:
	CLibraryData *m_pLibraryData;
	std::string m_strDLLName;
};
/*!
\brief 互斥类
*/
struct SMutexData;
class BASE_API CPPMutex {
public:
    CPPMutex();
    ~CPPMutex();

    /**
    * @brief 尝试锁定（非阻塞）
    * @return true:锁定成功, false:锁定失败.
    */
    type_bool tryLock();
    /**
    * @brief 锁定(阻塞)
    * @return void.
    */
    void lock(void);
    /**
    * @brief 解除锁定
    * @return void.
    */
    void unlock(void);

private:
    SMutexData *m_pMutexData;
};
/*!
\brief 自动锁
*/
class BASE_API CPPMutexLocker {
public:
    CPPMutexLocker(CPPMutex *Mutex, type_bool bLock = true);
    ~CPPMutexLocker();
    /**
    * @brief 尝试锁定（非阻塞）
    * @param[in] mseconds 超时时间。超时后退出。
    * @return true:锁定成功, false:锁定失败.
    */
    type_bool tryLock(type_int32 mseconds = 1000);

private:
    CPPMutex*	m_pMutex;
    type_bool   m_bLocked;
};

extern CPPMutex g_mutexTime;
/*!
\brief 线程处理类
*/
class CPPThreadData;
class BASE_API CPPThread {
public:
    enum ETHREAD_ALIVE_TIME {
        ETHREAD_ALIVE_MAXTIME = 5 * 1000, // 5分钟
    };
    // 构造函数
	CPPThread();
    // 析构函数
	virtual ~CPPThread();

    /**
    * @brief  启动线程
    * @param[in] pParam 启动参数，用来将应用数据传递给线程对象。
    * @return 无
    */
    bool start(void *pParam=NULL);
    /**
    * @brief  获取线程参数。
    * @return 线程参数
    */
    void* getParam() {
        return m_pParam;
    }

    /**
    * @brief  线程主循环函数。内部必须采用while(isWorking())作为主循环。
    * @return 无
    */
    virtual void run(void) = 0;
    
    /**
    * @brief  获取线程Id
    * @return 线程Id
    */
    int getThreadId();

    /**
    * @brief 线程是否正在运行
    * @return true:正在运行, false:线程已终止.
    */
    bool isWorking(void);
    /**
    * @brief	终止线程运行。该接口会等待线程正常退出，不会强制线程退出。
    * @return   void
    */
    virtual void exitThread(void);
    /**
    * @brief 线程状态是否正常。
    * @return true:正常, false:异常.
    */
    bool isAlive(void);
    /**
    * @brief 保持线程生命值。当派生类的主循环中可能出现耗时操作时，可以重写该接口，将m_nAlive设置为更大的值。
    * @return void.
    */
    void keepAlive();
    /**
    * @brief 获取线程最近报告的存活时间。
    * @return void.
    */
    type_time_t getAliveTime(void);
    /**
    * @brief 设置线程存活时间阈值。
    * @param[in] tAliveTimeLimit  存活时间阈值
    * @return void.
    */
    void setAliveTimeLimit(type_time_t tAliveTimeLimit = ETHREAD_ALIVE_MAXTIME) {
        m_tAliveTimeLimit = tAliveTimeLimit;
    }
    /**
    * @brief 获取线程存活时间阈值。
    * @return 存活时间阈值.
    */
    type_time_t getAliveTimeLimit() {
        return m_tAliveTimeLimit;
    }
protected:
    /**
    * @brief 获取线程内部数据对象
    * @return 线程内部数据对象.
    */
    CPPThreadData* getPrivateData(void);
    /**
    * @brief 关闭线程句柄
    * @return true:正在运行, false:线程已终止.
    */
    void closeThreadHandle(void);

private:
    /**
    * @brief 初始化线程Id
    * @return void
    */    
    void initializeThreadId(void);
private:
	CPPThreadData *m_pThreadData;
    CPPMutex m_mtxAliveTime;  // 用来保护m_tAlive的锁
    CPPMutex m_mtxRunning; // 用来保护成员变量的锁
    void *m_pParam;     // 应用层传入的参数。
    bool m_bWorking;	// 线程正在工作
    bool m_bFinished;	// 线程已停止工作
    type_time_t m_tAliveTimeLimit; // 线程存活时间阈值
    type_time_t m_tAlive;   // 线程最近存活时间
    friend CPPThreadData;
};

/*!
\brief 事件管理类
*/
class BASE_API CPPEventManager {
public:
    CPPEventManager();
    ~CPPEventManager();

    /**
    * @brief 向队列中添加事件
    * @return true:成功, false:失败.
    */
    bool addEvent(type_int32 nEventId);
    /**
    * @brief 清空事件队列
    * @return void.
    */
    void clearQueue();
    /**
    * @brief 从队列中获取一个事件Id
    * @return 取出的事件Id
    */
    type_int32 getOneEvent();
private:
    CPPMutex	m_mutex;            // 互斥锁
    std::queue<type_int32> m_eventsQueue;  // 事件队列
    type_int32 m_nEventCount;       // 队列中的事件个数
};
/*!
\brief 事件监听类
*/
class BASE_API CPPEventListener {
public:
    // 构造函数
    CPPEventListener();
    /**
    * @brief 构造函数
    * @param[in] pEventManager 事件管理对象指针
    */
    explicit CPPEventListener(CPPEventManager *pEventManager);

    /**
    * @brief 设置事件管理者对象
    * @return true:成功 false:失败
    */
    bool setEventManager(CPPEventManager *pEventManager);
    /**
    * @brief 等待事件
    * @param[in] nTimeout 超时时间，单位为毫秒。
    * @return 等到的事件Id
    */
    type_int32 waitForEvent(type_uint32 nTimeout=0);

private:
    type_int32	m_nEventCount;          // 事件个数
    type_int32	m_nEventType;			// 事件类型
    type_int32	m_nTimeout;				// 超时时间(毫秒)
    CPPEventManager *m_pEventManager;   // 事件管理对象指针
};

/*!
\brief 时间结构体
*/
struct STime {
    type_int32	nYear;
    type_int32	nMonth;
    type_int32	nDay;
    type_int32	nHour;
    type_int32	nMinute;
    type_int32	nSecond;
    type_int32	nFraction;
};
class BASE_API CPPTime {
public:
    enum ETimeFormat {
        ETF_SHOW_DATE = 0x01, // 只显示日期
        ETF_SHOW_TIME = 0x02, // 只显示时间
        ETF_SHOW_DATETIME = 0x03 // 显示日期、时间
    };
    CPPTime();
    CPPTime(const CPPTime& tmTime);
    CPPTime& operator = (const CPPTime& tmTime);
    ~CPPTime();

    /**
    * @brief 设置为当前时间
    * @return void.
    */
    void setCurrentTime(void);
    /**
    * @brief 设置为指定时间
    * @param[in] tm 指定时间，以秒为单位，从1900年开始计数。
    * @return true:成功 false:失败
    */
    bool setTime(type_time_t tm);
    /**
    * @brief 设置为指定时间
    * @param[in] nYear 年
    * @param[in] nMonth 月
    * @param[in] nDay 日
    * @param[in] nHour 时
    * @param[in] nMin 分
    * @param[in] nSec 秒
    * @return true:成功 false:失败。
    */
    bool setTime(int nYear, int nMonth, int nDay,
                    int nHour=0, int nMin=0, int nSec=0);
    /**
    * @brief 设置为指定时间
    * @param[in] pTimeString 指定时间字符串，格式为xxx
    * @return true:成功 false:失败
    */
    bool setTime(const char *pTimeString);
    /**
    * @brief 获取时间
    * @return 以秒为单位，从1900年开始计数。
    */
    type_time_t getTime(void) const;
    /**
    * @brief 把时间转换为本地时间。
    * @return 本地时间。
    */
    STime getLocalTime() const;
    /**
    * @brief 获取时间，以字符串方式返回。
    * @param[out] tTime 将获取到的时间保存到tTime中。
    * @return void。
    */
    std::string toStdString(ETimeFormat nTimeFormat = ETF_SHOW_DATE) const;
    /**
    * @brief 判断对象中存的时间是上午还是下午。
    * @return true:上午， false:下午。
    */
    bool isAM() const;
    /**
    * @brief 获取时间对象中的年份
    * @return 年份。
    */
    type_int32 getYear() const;
    /**
    * @brief 获取时间对象中的月份, 1表示1月份。
    * @return 月份。
    */
    type_int32 getMonth() const;
    /**
    * @brief 获取时间对象中的日期在月份中是第几天。day of month
    * @return 日。
    */
    type_int32 getDay() const;
    /**
    * @brief 获取时间对象中的小时。
    * @return 小时。
    */
    type_int32 getHour() const;
    /**
    * @brief 获取时间对象中的分钟。
    * @return 分钟。
    */
    type_int32 getMinute() const;
    /**
    * @brief 获取时间对象中的秒。
    * @return 秒。
    */
    type_int32 getSecond() const;
    /**
    * @brief 获取时间对象中的星期。比如，0=星期天，1=星期一，2=星期二，6=星期六
    * @return 星期。
    */
    type_int32 getWeek() const;
    /**
    * @brief 获取时间对象中的天数。
    * @return 当年中的第几天。比如返回1表示当年的第1天。
    */
    type_int32 getDayOfYear() const;
    /**
    * @brief 增加指定时间。
    * @param[in] nDays, 增加的天数
    * @param[in] nHours, 增加的小时数
    * @param[in] nMins, 增加的分钟数
    * @param[in] nSecs, 增加的秒数
    * @return void。
    */
    void addTime(type_int32 nDays, type_int32 nHours=0, type_int32 nMins=0, type_int32 nSecs=0);
    /**
    * @brief 是否与指定时间相等
    * @param[in] tmTime 指定时间
    * @return true:相等, false:不等。
    */
    bool operator == (const CPPTime& tmTime) const;
    /**
    * @brief 是否与指定时间不相等
    * @param[in] tmTime 指定时间
    * @return true:不相等, false:相等。
    */
    bool operator != (const CPPTime& tmTime) const;
    /**
    * @brief 是否比指定时间早
    * @param[in] tmTime 指定时间
    * @return true:比指定时间早, false:不比指定时间早。
    */
    bool operator < (const CPPTime& tmTime) const;
    /**
    * @brief 是否早于等于指定时间
    * @param[in] tmTime 指定时间
    * @return true:早于等于指定时间, false:比指定时间晚。
    */
    bool operator <= (const CPPTime& tmTime) const;
    /**
    * @brief 是否比指定时间晚
    * @param[in] tmTime 指定时间
    * @return true:比指定时间晚, false:不比指定时间晚。
    */
    bool operator > (const CPPTime& tmTime) const;
    /**
    * @brief 是否晚于等于指定时间
    * @param[in] tmTime 指定时间
    * @return true:晚于等于指定时间, false:比指定时间早。
    */
    bool operator >= (const CPPTime& tmTime) const;

private:
    type_time_t	m_nTime;
};

}
