﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。
\file: thread_linux.cpp
\brief 台线程类 Unix/Linux 平台实现
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/9
*/

#include <pthread.h>
#include <sys/syscall.h>
#include "base_class.h"
#include "service_api.h"

namespace ns_train {
extern "C" {static void *start_thread(void *t);}
class CPPThreadData {
public:
    CPPThreadData():m_nThreadID(0), m_nId(0) {
	}

    ~CPPThreadData() {
        closeHandle();
    }
    void closeHandle() {
        if (m_nThreadID != 0) {
            pthread_cancel(m_nThreadID); // 线程主循环中，需要调用pthread_testcancel()才能接收到pthread_cancel（）。
            m_nThreadID = 0;
        }
    }

    bool init(CPPThread *pThread) 	{
		pthread_attr_t attr;
		int ret;
		pthread_attr_init(&attr);
		pthread_attr_setinheritsched(&attr, PTHREAD_INHERIT_SCHED);
		pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
        ret = pthread_create(&m_nThreadID, &attr, start_thread, (void*)pThread);
		pthread_attr_destroy(&attr);
        if (ret == 0) {
			return true;
        }
        else {
            return false;
        }
	}
	
    int initThreadId() {
		m_nId = syscall(SYS_gettid);	
	}
    int getThreadId() {
		return m_nId;
    }
    static void start(CPPThread *pThread) {
		pThread->initializeThreadId();
		pThread->run();
        pThread->closeThreadHandle();
	}
private:
    pthread_t m_nThreadID;
	int m_nId;
};

extern "C" {
	static void *start_thread(void *t) 	{
        CPPThreadData::start(reinterpret_cast<CPPThread*>(t));
        return NULL;
	}
}

CPPThread::CPPThread() : m_pParam(NULL), m_bWorking(false),m_bFinished(true), m_tAliveTimeLimit(ETHREAD_ALIVE_MAXTIME) {
    m_pThreadData = new CPPThreadData;
}

CPPThread::~CPPThread() {
    if (m_pThreadData != NULL) {
        delete m_pThreadData;
    }
    m_pThreadData = NULL;
}

bool CPPThread::start(void *pParam) {
    if (pParam != NULL) 	{
        m_pParam = pParam;
	}
    {
        CPPMutexLocker locker(&m_mtxRunning);
        m_bFinished = false; // 防止线程的run()直接返回并且在run（）内将m_bFinished设置为true，因此需要先设置再调用init（）.
        m_bWorking = true;   // 线程的主循环中，以while(isWorking())进行判断，因此在启动线程之前，应将m_bWorking设置为true。
    }
    bool bOK = m_pThreadData->init(this);
    {
        CPPMutexLocker locker(&m_mtxRunning);
        m_bWorking = bOK;
    }
    if (bOK) {
        keepAlive();
    }
    return bOK;
}
bool CPPThread::isAlive(void) {
    CPPMutexLocker locker(&m_mtxAliveTime);
    type_time_t tCurrent = get_current_time();
    return ((tCurrent - m_tAlive) < m_tAliveTimeLimit) ? true : false;
}
void CPPThread::keepAlive() {
    ns_train::CPPMutexLocker locker(&m_mtxAliveTime);
    m_tAlive = ns_train::get_current_time();
}
type_time_t CPPThread::getAliveTime(void) {
    ns_train::CPPMutexLocker locker(&m_mtxAliveTime);
    return m_tAlive;
}
bool CPPThread::isWorking() {
    CPPMutexLocker locker(&m_mtxRunning);
    return m_bWorking;
}

void CPPThread::closeThreadHandle() {
    m_pThreadData->closeHandle();
    {
        CPPMutexLocker locker(&m_mtxRunning);
        m_bWorking = false;
        m_bFinished = true;
    }
}

void CPPThread::exitThread() {
    {
        CPPMutexLocker locker(&m_mtxRunning);
        m_bWorking = false;
    }
    while (!m_bFinished) { // 等待线程完全退出。
        api_sleep(10);
    }
    closeThreadHandle();
}
int CPPThread::getThreadId() {
    return m_pThreadData->getThreadId();
}
void CPPThread::initializeThreadId(void) {
	m_pThreadData->initThreadId(); // 必须在线程的run（）函数在一起运行，才能正确获取线程自身的id。
}
}
