﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: clientthread.h
\brief 客户端处理线程类定义。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang) 
\Date 2018/10
*/

#pragma once

#include "base_class.h"
#include <list>

struct s_IDValue {
    s_IDValue(type_uint32 id, type_double value);
    type_uint32 nId;
    type_double dValue;
};

namespace ns_train {
    class CPPSerial;
}

class CClientThread : public ns_train::CPPThread {
public:
    CClientThread();

    /**
    * @brief  线程主循环函数。内部必须采用while(isWorking())作为主循环。
    * @return 无
    */
    virtual void run() override;
    
    /**
    * @brief  添加命令：获取服务器端数据。采用异步方式，即先把命令压栈。
    * @param[in] uDataId 数据项Id
    * @return 无
    */    
    void addCommand_GetData(type_uint32 uDataId);
    /**
    * @brief  添加命令：更新服务器端数据。采用异步方式，即先把命令压栈。
    * @param[in] uDataId 数据项Id
    * @param[in] dValue 数据
    * @return 无
    */    
    void addCommand_SetData(type_uint32 uDataId, type_double dValue);
    /**
    * @brief  添加命令：获取服务器端的文件。
    * @param[in] str 文件名（不带路径，默认从该目录获取：$PROJECT_DEV_HOME/test/chapter07/ks07_07）
    * @return 无
    */
    void addCommand_GetFile(const std::string& str);
private:
    /**
    * @brief  将待发送数据进行组帧并发送
    * @return true:成功，false：失败。当写入串口失败时，将导致返回false。
    */
    bool makeFrameAndSend();
    /**
    * @brief  接收数据并解帧。
    * @param[in] pSerial 串口对象。
    * @return true:成功，false：失败。当写入串口失败时，将导致返回false。
    */
    bool recvAndParseFrame();
    /**
    * @brief  读取指定长度的数据
    * @param[in] nAllData 待读取的字节总数，含已经收到的字节数。
    * @return true：成功，false：未读完
    */
    bool readData(type_int32 nAllData);
    /**
    * @brief  获取帧类型。
    * @return 无
    */
    bool parseFrame_GetFrameType();
    /**
    * @brief  搜索同步字
    * @return 无
    */
    bool parseFrame_Synchronization();
    /**
    * @brief  获取帧长度。
    * @return 无
    */
    bool parseFrame_GetFrameLength();
    /**
    * @brief  获取单个模拟量数据帧(ESERVERFRAME_ONEANALOGDATA)的帧长度。
    * @return 无
    */
    bool parseFrame_GetFrameLength_OneAnalog();
    /**
    * @brief  获取模拟量数据帧(ESERVERFRAME_ANALOGDATAS)的帧长度。
    * @return 无
    */
    bool parseFrame_GetFrameLength_Analogs();
    /**
    * @brief  获取文件数据帧(ESERVERFRAME_FILE)的帧长度。
    * @return 无
    */
    bool parseFrame_GetFrameLength_File();
    /**
    * @brief  解析数据。
    * @return 无
    */
    bool parseFrame_Parse();
    /**
    * @brief  解析单个模拟量数据帧(ESERVERFRAME_ONEANALOGDATA)。
    * @return 无
    */
    bool parseFrame_OneAnalog();
    /**
    * @brief  解析模拟量数据帧(ESERVERFRAME_ANALOGDATAS)。
    * @return 无
    */
    bool parseFrame_Analogs();
    /**
    * @brief  解析文件数据帧(ESERVERFRAME_FILE)。
    * @return 无
    */
    bool parseFrame_File();
    /**
    * @brief  复位与接收数据相关的变量。
    * @return 无
    */
    void reset_receive();
    /**
    * @brief  复位与发送数据相关的变量。
    * @return 无
    */
    void reset_send();    
	/**
    * @brief  释放申请的内存
    * @return 无
    */
    void freeBuffer();
private:
    enum EFRAME_STATUS {
        EFRAME_SYNCHRONIZATION = 0, // 搜索同步字
        EFRAME_GET_FRAMETYPE,       // 搜索帧类型
        EFRAME_GET_FRAMELENGTH,     // 获取帧长度
        EFRAME_PARSEDATA,           // 解析数据
    };
    ns_train::CPPMutex m_mutex;     // 成员锁，用来保护对m_lstGetId、m_lstSetIdValue的访问。
    std::list<type_uint32> m_lstGetId;			// 待获取的“数据项Id”列表
    std::list<s_IDValue> m_lstSetIdValue;		// 待设置的”数据项Id、值“列表
    std::string m_strFileName;      // 文件名，不带路径。
    ns_train::CPPSerial* m_pSerial;             // 串口通信对象
    type_int32 m_nReceivedCount;				// 已经接收到的数据字节数
    EFRAME_STATUS m_eFrameStatus;               // 数据解帧状态
    type_uint8 m_byFrameType;					// 帧类型
    type_int32 m_nFrameLength;					// 帧长度
    type_uint32 m_u32StartId;					// 起始数据序号
    type_uint32 m_u32CurrentFrameDataCount;		// 本帧数据个数
    char *m_pReceiveBuf;                        // 接收缓冲区
    char *m_pSendBuf;                           // 发送缓冲区
};


