﻿/*!
* Copyright (C) 2018 女儿叫老白
* 版权所有。
* 代码仅用于课程《C++老鸟日记》的学习，请勿传播。
* 免责声明:代码不保证稳定性，请勿用作商业用途，否则后果自负。

\file: protocolobject.cpp
\brief 通信规约对象实现文件。
\author 女儿叫老白，代表作《Qt 5/PyQt 5实战指南》，微信公众号:软件特攻队(微信号:xingdianketang)
\Date 2018/10
*/
#include "ks07_08_server_protocolobject.h"

#include <iostream>
#include <string.h>

#include "base/basedll/base_class.h"
#include "base/basedll/base_file.h"
#include "com_serial.h"
#include "ks07_08/datavector.h"
#include "host_network_api.h"
#include "ks07_08/ks07_08_frametype.h"
#include "service_api.h"
#include "ks07_08/ks07_08_struct.h"

static const int c_nReceiveBufferLength = 12800; // 需要注意:确保内存够用，也就是能够至少容纳一帧接收到的数据。
static const int c_nSendBufferLength = 12800;	 // 需要注意:确保内存够用，也就是能够至少容纳一帧发送的数据。
												 //   另外，需要注意，如果发送缓冲区设置过大，会导致一帧数据太多，
												 //   可能导致被操作系统丢弃，进而导致接收端无法完整接收。
CServerProtocolObject::CServerProtocolObject(): m_n32FrameIndex(0), m_n64SendDataCount(0) {
	makeBuffer();
	reset_receive();
	reset_send();
}

CServerProtocolObject::~CServerProtocolObject() {
}

void CServerProtocolObject::reset_receive() {
	CProtocolObject::reset_receive();
	m_nReceivedCount = 0;
	m_byFrameType = static_cast<type_uint8>(ECLIENTFRAME_INVALID);
	m_eFrameStatus = EFRAME_SYNCHRONIZATION;
	m_nFrameLength = 0;
    m_u32StartId = 0;
    m_u32CurrentFrameDataCount = 0;
}

void CServerProtocolObject::reset_send() {
    m_bSendData = false;
    m_bSendFile = false;
	m_bWaitAck = false;
	m_n64SendDataCount = 0;
}

void CServerProtocolObject::makeBuffer() {
	m_pReceiveBuf = new char[c_nReceiveBufferLength];
	memset(m_pReceiveBuf, 0, c_nReceiveBufferLength * sizeof(char));
	m_pSendBuf = new char[c_nSendBufferLength];
	memset(m_pSendBuf, 0, c_nSendBufferLength * sizeof(char));
	m_pReSendBuf = new char[c_nSendBufferLength];
	memset(m_pReSendBuf, 0, c_nSendBufferLength * sizeof(char));
}

void CServerProtocolObject::setSerialObject(ns_train::CPPSerial* pSerial) {
	CProtocolObject::setSerialObject(pSerial);
	ns_train::s_serial_config config = pSerial->getSerialConfig();
	config.u32RecvBufferLength = c_nReceiveBufferLength;
	config.u32SendBufferLength = c_nSendBufferLength;
	if (pSerial->isOpen()) {
		close();
	}
	pSerial->setSerialConfig(config);
}

bool CServerProtocolObject::open() {
	if (NULL != m_pSerial) {
		return CProtocolObject::open();
	}
	else {
		return false;
	}
}

void CServerProtocolObject::recvAndParseFrame() {
	/*
	*	|同步字0x68	|   字节0     - 【1个字节】
	*	|帧类型		|   字节1     - 【1个字节】
	*	|帧序号		|   字节2~5   - 【4个字节】
	*	|...		|   不同的“帧类型”有不同的帧格式，请查看后续代码
	*/

	// 接收数据并解析
	switch (m_eFrameStatus) {
	case EFRAME_SYNCHRONIZATION:
		parseFrame_Synchronization();
		break;
	case EFRAME_GET_FRAMETYPE:
		parseFrame_GetFrameType();
		break;
	case EFRAME_GET_FRAMELENGTH:
		parseFrame_GetFrameLength();
		break;
	case EFRAME_PARSEDATA:
		parseFrame_Parse();
		break;
	default:
		reset_receive();
		return;
	}
}

void CServerProtocolObject::parseFrame_Synchronization() {
	reset_receive();
	const type_uint8 uchStart = 0x68;		// 同步字
	// 1. 设置需要读取的总字节数
	type_int32 nAllData = static_cast<int>(sizeof(uchStart)); // 【同步字】
    if (!readData(nAllData)) {
        return;
    }
	// 6. 当数据收全后，解析并保存数据、切换工作状态
	if (m_nReceivedCount == nAllData) { // 已经读取完毕
		if (m_pReceiveBuf[0] == 0x68) {
			m_eFrameStatus = EFRAME_GET_FRAMETYPE;		// 切换到下一个工作状态
		}
		else {
			reset_receive();
		}
	}
}

void CServerProtocolObject::parseFrame_GetFrameType() {
	const type_uint8 uchStart = 0x68;		// 同步字
	// 1. 设置需要读取的总字节数
	type_int32 nAllData = static_cast<int>(sizeof(uchStart) + sizeof(m_byFrameType)); // 【同步字】 + 【帧类型】
    if (!readData(nAllData)) {
        return;
    }
	// 6. 当数据收全后，解析并保存数据、切换工作状态
	if (m_nReceivedCount == nAllData) { // 已经读取完毕
		ks07_08_FrameHeader* pHeader = reinterpret_cast<ks07_08_FrameHeader*>(m_pReceiveBuf);
		pHeader->NetToHost();
		m_byFrameType = pHeader->byFrameType; // 保存【帧类型】
		m_eFrameStatus = EFRAME_GET_FRAMELENGTH;		// 切换到下一个工作状态
	}
}

void CServerProtocolObject::parseFrame_GetFrameLength() {
	switch (m_byFrameType) {
	case ECLIENTFRAME_ACK:
		parseFrame_GetFrameLength_Ack();
		break;
	case ECLIENTFRAME_SETDATA:
		parseFrame_GetFrameLength_SetData();
		break;
	case ECLIENTFRAME_GETDATA:
		parseFrame_GetFrameLength_GetData();
		break;
	case ECLIENTFRAME_GETFILE:
		parseFrame_GetFrameLength_GetFile();
		break;
	default:
		reset_receive();
		return;
	}
}

void CServerProtocolObject::parseFrame_GetFrameLength_Ack() {
	/*
	*	|同步字0x68		|   字节0     - 【1个字节】
	*	|帧类型			|   字节1     - 【1个字节，ECLIENTFRAME_ACK】
	*	|帧序号			|   字节2~5   - 【4个字节】
	*	|被确认的帧序号	|   字节6~9   - 【4个字节】
	*/
	// 设置需要读取的字节数。 需要把前面的字段都算上，然后得到【本帧数据个数】，从而计算帧长度。
	m_nFrameLength = sizeof(ks07_08_Client_Ack);
	m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
}

void CServerProtocolObject::parseFrame_GetFrameLength_SetData() {
	/*
	*	|同步字0x68	|   字节0		- 【1个字节】
	*	|帧类型		|   字节1		- 【1个字节，ECLIENTFRAME_SETDATA】
	*	|帧序号		|   字节2~5		- 【4个字节】
	*	|数据项Id	|   字节6~9		- 【4个字节】
	*	|数据值		|   字节10~17	- 【8个字节，double】
	*/

	// 设置需要读取的字节数。 需要把前面的字段都算上，然后得到【本帧数据个数】，从而计算帧长度。
	m_nFrameLength = sizeof(ks07_08_Client_SetData);
	m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
}

void CServerProtocolObject::parseFrame_Parse() {
	switch (m_byFrameType) {
	case ECLIENTFRAME_ACK:
		parseFrame_Ack();
		break;
	case ECLIENTFRAME_SETDATA:
		parseFrame_SetData();
		break;
	case ECLIENTFRAME_GETDATA:
		parseFrame_GetData();
		break;
	case ECLIENTFRAME_GETFILE:
		parseFrame_GetFile();
		break;
	default:
		reset_receive();
		return;
	}
}

void CServerProtocolObject::parseFrame_Ack() {
	/*
	*	|同步字0x68				|   字节0     - 【1个字节】
	*	|帧类型					|   字节1     - 【1个字节，ECLIENTFRAME_ACK】
	*	|数据项被确认的帧序号	|   字节2~5   - 【4个字节】
	*/
	CDataVector* pDataVector = CDataVector::instance();
	if (NULL == pDataVector) {
		return;
	}
    if (!readData(m_nFrameLength)) {
        return;
    }
    if (m_nReceivedCount < m_nFrameLength) {
		return;
    }
	ks07_08_Client_Ack* pAck = reinterpret_cast<ks07_08_Client_Ack*>(m_pReceiveBuf);
	pAck->NetToHost();
	if (pAck->n32FrameIndexAck == m_n32FrameIndex) { // 收到正确的确认帧
		m_bWaitAck = false;
		m_bResend = false;
		m_nResendLength = 0;
		m_nResendCount = 0;
		std::cout << std::endl << "收到客户端确认帧: 序号 = " << m_n32FrameIndex << std::endl;
	}
	else {
		std::cout << std::endl << "收到客户端确认帧出错: 序号 = " 
					<< pAck->n32FrameIndexAck << ",应该收到的确认帧序号="<< m_n32FrameIndex << std::endl;

	}
	reset_receive();
}
void CServerProtocolObject::parseFrame_SetData() {
	/*
	*	|同步字0x68	|   字节0     - 【1个字节】
	*	|帧类型		|   字节1     - 【1个字节，ECLIENTFRAME_SETDATA】
	*	|帧序号		|   字节2~5	  - 【4个字节】
	*	|数据项Id   |   字节6~9   - 【4个字节】
	*	|数据值		|   字节10~17 - 【8个字节，double】
	*/
	CDataVector* pDataVector = CDataVector::instance();
	if (NULL == pDataVector) {
		return;
	}
    if (!readData(m_nFrameLength)) {
        return;
    }
    if (m_nReceivedCount < m_nFrameLength) {
		return;
    }
	ks07_08_Client_SetData* pSetData = reinterpret_cast<ks07_08_Client_SetData*>(m_pReceiveBuf);
	pSetData->NetToHost();
	type_uint8 byCheckSum = pSetData->calculateChecksum();
	if (byCheckSum != pSetData->checkSum) {
		std::cout << std::endl << "收到客户端数据: 校验错误." << std::endl;
		reset_receive();
		return;
	}

	if (pSetData->data.uDataId < pDataVector->getSize()) {
		pDataVector->setData(pSetData->data.uDataId + 1, pSetData->data.dValue);
	}
	reset_receive();
	std::cout << std::endl << "收到客户端数据: dataId = " << pSetData->data.uDataId << "（序号从0开始）, value = " << pSetData->data.dValue << std::endl;
}

void CServerProtocolObject::parseFrame_GetFrameLength_GetData() {
	/*
	*	|同步字0x68	|   字节0     - 【1个字节】
	*	|帧类型		|   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
	*	|帧序号		|   字节2~5	  - 【4个字节】
	*	|数据项Id  	|   字节6~9   - 【4个字节】
	*/

	// 设置需要读取的字节数。 需要把前面的字段都算上，然后得到【本帧数据个数】，从而计算帧长度。
	m_nFrameLength = sizeof(ks07_08_Client_GetData);
	m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态
}

void CServerProtocolObject::parseFrame_GetFrameLength_GetFile() {
	/*
	*	|同步字0x68	|   字节0     - 【1个字节】
	*	|帧类型		|   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
	*	|帧序号		|   字节2~5	  - 【4个字节】
	*   |文件名     |   字节6-    - 【MAX_FILE_NAME_LENGTH】
    */
	m_nFrameLength = sizeof(ks07_08_Client_GetFile); 
	m_eFrameStatus = EFRAME_PARSEDATA; // 切换到下一个工作状态  
}

void CServerProtocolObject::parseFrame_GetData() {
	/*
	*	|同步字0x68	|   字节0     - 【1个字节】
	*	|帧类型		|   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
	*	|帧序号		|   字节2~5	  - 【4个字节】
	*	|数据项Id	|   字节6~9   - 【4个字节】
	*/
	CDataVector* pDataVector = CDataVector::instance();
	if (NULL == pDataVector) {
		return;
	}
    if (!readData(m_nFrameLength)) {
        return;
    }
    if (m_nReceivedCount < m_nFrameLength) {
        return;
    }

	reset_receive();   // 先复位，再赋值，否则数据会被冲掉
	ks07_08_Client_GetData* pGetData = reinterpret_cast<ks07_08_Client_GetData*>(m_pReceiveBuf);
	pGetData->NetToHost();
	m_u32SendDataId = pGetData->uDataId;
	m_bSendData = true;
	std::cout << std::endl << "收到客户端getdata命令: dataId = " << m_u32SendDataId << "（序号从0开始）。 " << std::endl;
}

void CServerProtocolObject::parseFrame_GetFile() {
	/*
	*	|同步字0x68		|   字节0     - 【1个字节】
	*	|帧类型			|   字节1     - 【1个字节，ECLIENTFRAME_GETDATA】
	*	|帧序号			|   字节2~5	  - 【4个字节】
	*   |文件名			|   字节6-    - 【MAX_FILE_NAME_LENGTH】
    */
	CDataVector* pDataVector = CDataVector::instance();
	if (NULL == pDataVector) {
		return;
	}
    if (!readData(m_nFrameLength)) {
        return;
    }
    if (m_nReceivedCount < m_nFrameLength) {
        return;
    }
	reset_receive();   // 先复位，再赋值，否则数据会被冲掉
	ks07_08_Client_GetFile* pGetFile = reinterpret_cast<ks07_08_Client_GetFile*>(m_pReceiveBuf);
	pGetFile->NetToHost();
	//m_strFileName = reinterpret_cast<char*>(&pGetFile->u32FileNameLength+sizeof(pGetFile->u32FileNameLength));
	m_strFileName = pGetFile->szFileName;
	m_bSendFile = true;
	std::cout << std::endl << "收到客户端getfile命令: 文件名 = " << m_strFileName << std::endl;

}

void CServerProtocolObject::makeFrameAndSend() {
	// 判断是否超时重发 
	if (m_bWaitAck) {
		ns_train::CPPTime tmNow;
		tmNow.setCurrentTime();
		if ((tmNow.getTime() - m_tmBeginWaitAck.getTime()) >= waitAckTime()) {
			m_bResend = true;
		}
	}

	reSend();	// 首先执行重发操作

	makeFrameAndSend_OneAnalog(); // 根据客户端请求，返回单个模拟量数据

	makeFrameAndSend_File(); // 根据客户端请求，返回文件

	ns_train::CPPTime tmNow;
	tmNow.setCurrentTime();
	if ((tmNow.getTime() - m_tmLastSendAllData.getTime()) > 5) { // 每5秒发送一次全部模拟量数据
		makeFrameAndSend_Analogs();
	}
}

void CServerProtocolObject::makeFrameAndSend_OneAnalog() {
	/*
	*	|同步字0x68|   字节0     - 【1个字节】
	*	|帧类型    |   字节1     - 【1个字节，ESERVERFRAME_ONEANALOGDATA】
	*	|帧序号	   |   字节2~5	 - 【4个字节】
	*	|数据项Id  |   字节6~9   - 【4个字节】
	*	|数据值    |   字节10~17 - 【8个字节，double】
	*/
	if (!m_bSendData) {
		return;
	}
	CDataVector* pDataVector = CDataVector::instance();
	if (NULL == pDataVector) {
		return;
	}
	ks07_08_Server_OneAnalogData* pAnalog = reinterpret_cast<ks07_08_Server_OneAnalogData*>(m_pSendBuf);
	pAnalog->header.byStart = 0x68; // 同步字
	pAnalog->header.byFrameType = static_cast<type_uint8>(ESERVERFRAME_ONEANALOGDATA);
	m_n32FrameIndex++;
	pAnalog->header.n32FrameIndex = m_n32FrameIndex;
	m_bSendData = false; // 把标志放倒。
	type_double dValue = 0.0;
	int nSend = 0;
	if (m_u32SendDataId < pDataVector->getSize()) {
		pDataVector->getData(m_u32SendDataId + 1, dValue);
		std::cout << std::endl << "发送单个数据给客户端: dataId = " << m_u32SendDataId << "（序号从0开始）, value = " << dValue << std::endl;
		pAnalog->data.uDataId = m_u32SendDataId;
		pAnalog->data.dValue = dValue;
		nSend += sizeof(ks07_08_Server_OneAnalogData);
		pAnalog->checkSum = pAnalog->calculateChecksum();
		pAnalog->HostToNet();
		int nWrite = m_pSerial->write(m_pSendBuf, nSend);
		m_u32SendDataId = 0;
		if (nWrite < 0) {
			m_pSerial->close();
			return;
		}
	}
}

void CServerProtocolObject::makeFrameAndSend_Analogs() {
	/*
	*	|同步字0x68		|   【1个字节】- 字节0
	*	|帧类型			|   【1个字节】- 字节1
	*	|帧序号			|   【4个字节】- 字节2~5
	*	|起始数据序号	|   【4个字节】- 字节6~9
	*	|本帧数据个数	|   【4个字节】- 字节10~13
	*	|1个数据		|   【8个字节】- 字节14~21
	*	|...			|
	*	|1个数据		|   【8个字节】- 字节n~n+7
	*/
	if (m_bSendFile) {
		return; // 正在发送文件，暂时不发送数据
	}
	CDataVector* pDataVector = CDataVector::instance();
	if (NULL == pDataVector) {
		return;
	}
	// 1. 定义通信数据帧中各个字段对应的变量。请注意，要保证变量的数据类型跟通信帧中对应的字段完全一致。
	bool bError = false;
	ks07_08_Server_AnalogDatas* pAnalogDatas = reinterpret_cast<ks07_08_Server_AnalogDatas*>(m_pSendBuf);
	type_double dValue = 0.0;		// 数据的值

	// 2. 计算该缓冲区能容纳多少有效数据
	type_uint8 checkSum = 0;            // 校验码
	const type_int32 nFrameMaxDataCount = (c_nSendBufferLength - sizeof(ks07_08_Server_AnalogDatas)-sizeof(checkSum)) / sizeof(dValue); // 一帧最多可发送的数据个数

	// 3. 定义相关变量。
	type_uint32 nAllDataCount = pDataVector->getSize();	// 待发送数据总个数
	type_uint32 nSendDataCount = 0;		// 已经发送的数据个数
	type_uint32 u32StartDataId = 0;		// 起始数据序号
	type_uint32 u32CurrentFrameDataCount = 0;   // 本帧数据个数
	type_int32 nFrameLength = 0;        // 本帧数据总长度
	type_uint8 *pBuf = NULL;            // 发送缓冲区游标指针

	while (nSendDataCount < nAllDataCount) { // 数据都发出去了吗？
		pAnalogDatas->header.byStart = 0x68;		// 同步字
		pAnalogDatas->header.byFrameType = type_uint8(ESERVERFRAME_ANALOGDATAS);	// 帧类型
		//m_n32FrameIndex++;
		pAnalogDatas->header.n32FrameIndex = 0;
		pAnalogDatas->u32StartDataId = u32StartDataId;	// 起始数据序号

		if ((nAllDataCount - nSendDataCount) < nFrameMaxDataCount) {
			u32CurrentFrameDataCount = nAllDataCount - nSendDataCount;
		}
		else {
			u32CurrentFrameDataCount = nFrameMaxDataCount;
		}
		pAnalogDatas->u32CurrentFrameDataCount = u32CurrentFrameDataCount;	// 本帧数据个数

		nFrameLength = static_cast<int>(sizeof(ks07_08_Server_AnalogDatas) + u32CurrentFrameDataCount * sizeof(dValue)); // 本帧数据长度

		pBuf = reinterpret_cast<type_uint8*>(m_pSendBuf + sizeof(ks07_08_Server_AnalogDatas)); // 将指针移动到数据区
		for (type_uint32 idx = 0; idx < u32CurrentFrameDataCount; idx++) {
			pDataVector->getData(idx + 1+u32StartDataId, dValue);
			*(reinterpret_cast<type_double*>(pBuf)) = ns_train::HostToNetwork(dValue);	// 数据
			pBuf += sizeof(dValue);
		}
		u32StartDataId += pAnalogDatas->u32CurrentFrameDataCount;
		nFrameLength += sizeof(checkSum);
		*pBuf = checkSum = pAnalogDatas->calculateChecksum(nFrameLength);
		pAnalogDatas->HostToNet(); // 转换字节序。在此行代码之后，不可以继续使用 pAnalogDatas 中的变量。
		// 数据组帧完毕，开始发送。
		int nWrite = m_pSerial->write(m_pSendBuf, nFrameLength);
		if (nWrite > 0) {
			nSendDataCount += u32CurrentFrameDataCount; // 更新已发送数据个数。
		}
		else {
			m_pSerial->close();
			bError = true;
			return;
		}
		// ns_train::ii_sleep(1); // 如果数据量比较多，导致循环次数太多，可能导致CPU占用率高，那么可以用睡眠释放CPU。
	}
	m_tmLastSendAllData.setCurrentTime();
}

void CServerProtocolObject::makeFrameAndSend_File() {
	/*
	*	|同步字0x68               	|   字节0		- 【1个字节】
	*	|帧类型                   	|   字节1		- 【1个字节】
	*	|帧序号						|   字节2~5		- 【4个字节】
	*   |本帧数据总长度				|	字节6~9		- 【4个字节】
	*	|文件名                   	|   字节10~73	- 【MAX_FILE_NAME_LENGTH】
	*	|是否最后一帧              	|   字节74		- 【1个字节】
	*	|本帧数据在文件中的起始偏移量|  字节75~82	- 【8个字节】
	*	|本帧文件内容长度           |   字节83~86	- 【4个字节】
	*	|本帧文件内容              	|   字节87~		- 【N个字节】
	*   |校验码						|   字节87+N	- 【1个字节】
	*/
	if (!m_bSendFile) {
		return;
	}
	if (m_bWaitAck) {
		return;
	}
	// 1. 定义通信数据帧中各个字段对应的变量。请注意，要保证变量的数据类型跟通信帧中对应的字段完全一致。
	ks07_08_Server_File* pFile = reinterpret_cast<ks07_08_Server_File*>(m_pSendBuf);

   // 2. 计算该缓冲区能容纳多少有效数据
	type_uint8 byCheckSum = 0;	// 校验码
	const type_int32 nFrameMaxDataCount = c_nSendBufferLength - sizeof(ks07_08_Server_File)-sizeof(byCheckSum); // 一帧最多可发送的数据个数

	// 3. 定义相关变量。
	ns_train::CPPFile file;
	type_int64 nAllDataCount = 0;	// 待发送数据总个数
	//type_int64 nSendDataCount = 0;	// 已经发送的数据个数
	type_int32 nFrameLength = 0;    // 本帧数据总长度
	type_uint32 u32CurrentFrameDataCount = 0;   // 本帧数据个数

	std::string strFileName;
	std::string strDirectory = ns_train::getDirectory("$PROJECT_DEV_HOME/test/chapter07/ks07_08/");
	ns_train::mkDir(strDirectory);
	strFileName = strDirectory + "/" + m_strFileName;
	strFileName = ns_train::getPath(strFileName);
	file.setFileName(strFileName);
	bool bOk = file.open("rb");
	if (!bOk) {
		reset_send();
		return;
	}
	type_size fileSize = file.getSize();
	strcpy(pFile->szFileName, m_strFileName.c_str()); // 文件名，不带路径。路径默认为：$PROJECT_DEV_HOME/test/chapter07/ks07_08
	pFile->byLastFrame = 0;     // 是否最后一帧
	pFile->u64Offset = 0;       // 本帧数据在文件中的起始偏移量
	pFile->u32DataLength = 0;   // 本帧文件内容长度

	nAllDataCount = static_cast<int>(fileSize);
	type_size sizeRead = 0;
	type_size nOffset = file.seekToPosition(m_n64SendDataCount, SEEK_SET); // 将文件游标移动到指定位置
	nOffset = file.currentPosition();
	type_int64 nFrameCount = (nAllDataCount+ nFrameMaxDataCount-1)/ nFrameMaxDataCount;
	std::cout << std::endl << "整个文件一共拆分为："<< nFrameCount << "帧" <<std::endl;
	if (m_n64SendDataCount < nAllDataCount) { // 数据都发出去了吗？
		pFile->header.byStart = 0x68;
		pFile->header.byFrameType = static_cast<type_uint8>(ESERVERFRAME_FILE);
		m_n32FrameIndex++;
		pFile->header.n32FrameIndex = m_n32FrameIndex;
		strcpy(pFile->szFileName, m_strFileName.c_str()); // 文件名，不带路径。路径默认为：$PROJECT_DEV_HOME/test/chapter07/ks07_08
		if ((nAllDataCount - m_n64SendDataCount) < nFrameMaxDataCount) {
			u32CurrentFrameDataCount = static_cast<type_uint32>(nAllDataCount - m_n64SendDataCount);// 本帧文件内容长度
			pFile->byLastFrame = 1; // 是否最后一帧
		}
		else {
			u32CurrentFrameDataCount = nFrameMaxDataCount;// 本帧文件内容长度
			pFile->byLastFrame = 0; // 是否最后一帧
		}
		pFile->u32DataLength = u32CurrentFrameDataCount;
		pFile->u64Offset = m_n64SendDataCount;   // 本帧数据在文件中的起始偏移量
		nFrameLength = pFile->u32FrameLength = sizeof(ks07_08_Server_File) + pFile->u32DataLength+sizeof(byCheckSum); // 本帧数据总长度

		sizeRead = file.read(m_pSendBuf + sizeof(ks07_08_Server_File), u32CurrentFrameDataCount); // 读取文件
		nOffset = file.currentPosition();
		if (sizeRead != pFile->u32DataLength) {
			return;
		}

		byCheckSum = pFile->calculateChecksum(nFrameLength);
		*(m_pSendBuf + sizeof(ks07_08_Server_File)+ pFile->u32DataLength) = byCheckSum;
		pFile->HostToNet(); // 转换字节序。在此行代码之后，不可以继续使用 pAnalogDatas 中的变量。
		// 数据组帧完毕，开始发送。
		int nWrite = 0;
		int nSend = 0;
		int nCurrentFrameSend = 0;

		// 设置重发数据
		memcpy(m_pReSendBuf, m_pSendBuf, nFrameLength);
		m_nResendLength = nFrameLength;

		for (nCurrentFrameSend = 0; nCurrentFrameSend < nFrameLength;) {
			nSend = nFrameLength - nCurrentFrameSend;
			nWrite = m_pSerial->write(m_pSendBuf + nCurrentFrameSend, nSend);
			
			if (nWrite < 0) {
				file.close();
				m_pSerial->close();
				reset_send();
				return;
			}
			else {
				nCurrentFrameSend += nWrite;
			}
		}
		m_n64SendDataCount += u32CurrentFrameDataCount;
		m_bWaitAck = true; // 需要等待客户端的Ack。
		m_bResend = false; // 复位“重发标志”
		m_tmBeginWaitAck.setCurrentTime(); // 更新“开始等待时间”
	}
	file.close();
	if (pFile->byLastFrame) {
		m_bSendFile = false;
		std::cout << "向客户端发送文件结束。帧序号为:"<< m_n32FrameIndex<< ",文件名为:" << strFileName << std::endl;
		m_n64SendDataCount = 0;
	}
	else {
		std::cout << "向客户端发送文件。帧序号为:" << m_n32FrameIndex
					<< ", 文件名为:" << strFileName 
					<< ", 已发送：" << m_n64SendDataCount 
					<< "字节。剩余"<< (nAllDataCount -m_n64SendDataCount)<< std::endl;
	}
}

void CServerProtocolObject::reSendOverLimit() { // 重发次数超限
	m_bSendFile = false; 
	m_n64SendDataCount = 0;
}